use serde::{Serialize, Deserialize};
use validator::Validate;
use diesel::PgConnection;
use diesel::r2d2::ConnectionManager;
use diesel::r2d2::Pool;

use crate::schema::user::{unique_email, unique_username};

#[derive(Serialize, Deserialize, Clone)]
pub struct Login {
    pub username: String,
    pub password: String
}

pub struct RegisterContext<'v_a> {
    pub pool: &'v_a Pool<ConnectionManager<PgConnection>>
}

#[derive(Serialize, Deserialize, Clone, Validate)]
#[validate(context = RegisterContext::<'v_a>)]
pub struct Register {
    #[validate(
        length(min = 1),
        custom(function = "unique_username", use_context)
    )]
    pub username: String,
    #[validate(
        email,
        custom(function = "unique_email", use_context)
    )]
    pub email: String,
    #[validate(length(min = 10), must_match(other = "password_repeat", ))]
    pub password: String,
    pub password_repeat: String,
}

#[derive(Serialize, Deserialize, Clone)]
pub struct GamenightPost {
    pub name: String,
    pub datetime: String
}

#[derive(Serialize, Deserialize, Clone)]
pub struct GamenightGet {
    pub id: String
}